Ext.namespace('Zarafa.widgets');

/**
 * @class Zarafa.widgets.SenderInfoWidget
 * @extends Zarafa.core.ui.widget.Widget
 */
Zarafa.widgets.SenderInfoWidget = Ext.extend(Zarafa.core.ui.widget.Widget, {

	/**
	 * The store in which the Twitter messages will be shown
	 * @property
	 * @type Ext.data.ArrayStore
	 */
	store : undefined,

	/**
	 * The record which currently selected by the context model.
	 * @property
	 * @type Zarafa.core.data.MAPIRecord
	 */
	record : undefined,

	/**
	 * @constructor
	 * @param {Object} config Configuration object
	 */
	constructor : function(config)
	{
		config = config || {};

 		this.store = new Ext.data.ArrayStore({
			// store configs
			autoDestroy: true,
			storeId: 'twidget',
			idIndex: 0,  
			fields: [
				'image',
				'sender',
				'user',
				'text'
			],
			data: [ ]
		});

		Ext.applyIf(config, { 
			name : 'senderinfowidget',
			title : _('Twidget'),
			height : 260,
			layout : 'fit',
			items : [{
				xtype: 'grid',
				disableSelection : true,
				store: this.store,
				border: true,
				sm: new Ext.grid.RowSelectionModel({
					singleSelect: true
				}),
				viewConfig: {
					deferEmptyText: false,
					emptyText: '<div class="emptytext">' + _('No sender info found.') + '</div>',
					forceFit: true,
					enableRowBody: true,
					getRowClass: function(record, rowIndex, rp, ds){
						rp.body = record.get('text');
						return 'x-grid3-row-expanded';
					}
				},
				colModel : new Ext.grid.ColumnModel({
					columns: [{
						header: '',
						dataIndex: 'image',
						editable: false,
						menuDisabled: true,
						renderer: function(value) {
							return '<img style="width: 16px; height: 16px" src="' + value + '">';
						},
						width: 50
					},{
						header: _('Username'),
						dataIndex: 'user',
						editable: false,
						menuDisabled: true,
						renderer: function(value) {
							return '<b>@' + value + '</b>';
						},
						width: 200
					},{
						header: _('Sender'),
						dataIndex: 'sender',
						editable: false,
						menuDisabled: true,
						width: 200
					}]
				})
			}]
		});

		Zarafa.widgets.SenderInfoWidget.superclass.constructor.call(this, config);

		var context = container.getContextByName('mail');
		if (context) {
			this.mon(context.getModel(), 'previewrecordchange', this.onPreviewRecordChange, this);
		}
	},

	/**
	 * Event handler which is fired when the {@link Zarafa.core.ContextModel Context Model} fires the
	 * {@link Zarafa.core.ContextModel#previewrecordchange} event. This will clear the {@link #store}
	 * and ensure that the new associated twitter messages are loaded.
	 * @param {Zarafa.core.ContextModel} contextModel The model which fired the event
	 * @param {Ext.data.Record} record The record which was selected
	 * @private
	 */
	onPreviewRecordChange : function(contextModel, record)
	{
		// In case we already had a previewed record, unhook the events
		if (this.record) {
			var store = this.record.getStore();
			if (store) {
				this.mun(store, 'open', this.onOpen, this);
			}
		}

		// Remove all associated twitter messages
		this.store.removeAll();

		// Update the current record
		this.record = record;
		if (!this.record) {
			return;
		}

		if (!record.isOpened()) {
			this.mon(record.getStore(), 'open', this.onOpen, this);
			record.open();
		} else {
			this.loadRecord(record);
		}
	},

	/**
	 * Event handler which is fired when the {@link #record} has been opened
	 * by the store. This will call {@link #loadRecord}.
	 * @param {Ext.data.Store} store The store which fired the event
	 * @param {Ext.data.Record} record The opened record
	 * @private
	 */
	onOpen : function(store, record)
	{
		if (this.record && this.record.equals(record)) {
			this.mun(store, 'open', this.onOpen, this);
			this.loadRecord(record);
		}
	},

	/**
	 * Load the associated twitter messages for the given record.
	 * This will send a request using JSONP to twitter to search for
	 * all associated messages
	 * @param {Ext.data.Record} record The record to load
	 * @private
	 */
	loadRecord : function(record)
	{
		var body = record.getBody(false);
		var re = new RegExp("twitter.com/(\\w+)");
		var twitter = undefined;

		var match = re.exec(body);
		if (Ext.isEmpty(match)) {
			re = new RegExp("^(.*?\\W)?@(\\w{1,15})\\b");
			match = re.exec(body);
			if (!Ext.isEmpty(match)) {
				twitter = match[2];
			}
		} else {
			twitter = match[1];
		}
		
		if (twitter !== undefined) {
			Ext.ux.JSONP.request({
				url : 'http://search.twitter.com/search.json',
				params: {
					q : '@' + twitter
				},
				success : this.onTwitterUpdate,
				callbackKey : 'callback',
				scope : this
			});
		}
	},

	/**
	 * Callback function from JSONP as provided by {@link #loadRecord},
	 * this will handle the result from Twitter to initialize the {@link #store}
	 * to show the new messages.
	 * @param {Object} data The data as received from twitter
	 * @private
	 */
	onTwitterUpdate : function(data)
	{
		Ext.each(data.results, function(result, i) {
			var r = new this.store.recordType({
				user: result.from_user,
				sender: result.from_user_name, 
				text: result.text,
				image: Ext.isSecure ? result.profile_image_url_https : result.profile_image_url
			}, i);
			this.store.add([r]);
		}, this);
	}
});

Zarafa.onReady(function() {
	container.registerWidget(Zarafa.widgets.SenderInfoWidget, 'twidget', _('Twidget'), 'plugins/twidget/resources/images/twidget.png');
});
